import os
import sys

sys.path.insert(0, os.path.dirname(__file__))

from dotenv import load_dotenv, find_dotenv
load_dotenv(find_dotenv())

from mcp_server import mcp
import tools  # noqa: F401  - registers tools on mcp

def main():
    mcp.run(transport=os.getenv("TRANSPORT", "stdio"))

if __name__ == "__main__":
    main()



from typing import Optional
from mcp_server import mcp, client
from utils.client import PatreonAPIError
from models import (
    User, Campaign, Tier, Member, Post,
    CampaignDetail, MemberPage, PostPage,
    parse_list, parse_single,
)


def _handle_error(e: Exception) -> str:
    if isinstance(e, PatreonAPIError):
        return str(e)
    if isinstance(e, ValueError):
        return str(e)
    return "Unexpected error communicating with Patreon API"


@mcp.tool()
def fetch_identity() -> User:
    """Fetch the authenticated Patreon user's profile."""
    try:
        response = client.get_identity()
        return parse_single(response, User)
    except Exception as e:
        raise RuntimeError(_handle_error(e))


@mcp.tool()
def fetch_campaigns() -> list[Campaign]:
    """Fetch all campaigns for the authenticated creator."""
    try:
        response = client.get_campaigns()
        campaigns, _ = parse_list(response, Campaign)
        return campaigns
    except Exception as e:
        raise RuntimeError(_handle_error(e))


@mcp.tool()
def fetch_campaign(campaign_id: str) -> CampaignDetail:
    """
    Fetch details for a specific campaign including tiers.

    Args:
        campaign_id: The campaign ID to fetch (numeric)
    """
    try:
        response = client.get_campaign(campaign_id)
        campaign = parse_single(response, Campaign)
        included = response.get("included", [])
        tiers = [
            Tier(id=t["id"], **t.get("attributes", {}))
            for t in included if t.get("type") == "tier"
        ]
        return CampaignDetail(campaign=campaign, tiers=tiers)
    except Exception as e:
        raise RuntimeError(_handle_error(e))


@mcp.tool()
def fetch_members(campaign_id: str, cursor: Optional[str] = None) -> MemberPage:
    """
    Fetch members/patrons of a campaign with pagination. Returns up to 100 members per page.
    Pass next_cursor from the response as cursor to get the next page.

    Args:
        campaign_id: The campaign ID (numeric)
        cursor: Pagination cursor for next page (from previous response)
    """
    try:
        response = client.get_members(campaign_id, cursor)
        members, next_cursor = parse_list(response, Member)
        return MemberPage(members=members, next_cursor=next_cursor)
    except Exception as e:
        raise RuntimeError(_handle_error(e))


@mcp.tool()
def fetch_posts(campaign_id: str, cursor: Optional[str] = None) -> PostPage:
    """
    Fetch posts from a campaign with pagination. Returns up to 20 posts per page.
    Pass next_cursor from the response as cursor to get the next page.

    Args:
        campaign_id: The campaign ID (numeric)
        cursor: Pagination cursor for next page (from previous response)
    """
    try:
        response = client.get_posts(campaign_id, cursor)
        posts, next_cursor = parse_list(response, Post)
        return PostPage(posts=posts, next_cursor=next_cursor)
    except Exception as e:
        raise RuntimeError(_handle_error(e))


@mcp.tool()
def fetch_post(post_id: str) -> Post:
    """
    Fetch a specific post by ID.

    Args:
        post_id: The post ID to fetch (numeric)
    """
    try:
        response = client.get_post(post_id)
        return parse_single(response, Post)
    except Exception as e:
        raise RuntimeError(_handle_error(e))



from pydantic import BaseModel, ConfigDict
from typing import Optional


class User(BaseModel):
    model_config = ConfigDict(extra="ignore")
    id: str
    full_name: Optional[str] = None
    email: Optional[str] = None
    image_url: Optional[str] = None
    about: Optional[str] = None
    url: Optional[str] = None
    created: Optional[str] = None


class Campaign(BaseModel):
    model_config = ConfigDict(extra="ignore")
    id: str
    creation_name: Optional[str] = None
    patron_count: Optional[int] = None
    pledge_url: Optional[str] = None
    published_at: Optional[str] = None
    url: Optional[str] = None
    vanity: Optional[str] = None
    is_monthly: Optional[bool] = None
    created_at: Optional[str] = None
    image_url: Optional[str] = None
    summary: Optional[str] = None
    one_liner: Optional[str] = None
    pay_per_name: Optional[str] = None


class Tier(BaseModel):
    model_config = ConfigDict(extra="ignore")
    id: str
    title: Optional[str] = None
    amount_cents: Optional[int] = None
    description: Optional[str] = None
    published: Optional[bool] = None
    patron_count: Optional[int] = None


class Member(BaseModel):
    model_config = ConfigDict(extra="ignore")
    id: str
    full_name: Optional[str] = None
    patron_status: Optional[str] = None
    pledge_cadence: Optional[int] = None
    lifetime_support_cents: Optional[int] = None
    currently_entitled_amount_cents: Optional[int] = None
    last_charge_date: Optional[str] = None
    last_charge_status: Optional[str] = None
    will_pay_amount_cents: Optional[int] = None
    is_follower: Optional[bool] = None
    tiers: list[str] = []
    user_name: Optional[str] = None


class Post(BaseModel):
    model_config = ConfigDict(extra="ignore")
    id: str
    title: Optional[str] = None
    content: Optional[str] = None
    is_paid: Optional[bool] = None
    is_public: Optional[bool] = None
    published_at: Optional[str] = None
    url: Optional[str] = None
    embed_data: Optional[dict] = None
    embed_url: Optional[str] = None
    app_id: Optional[str] = None
    app_status: Optional[str] = None


class CampaignDetail(BaseModel):
    model_config = ConfigDict(extra="ignore")
    campaign: Campaign
    tiers: list[Tier] = []


class MemberPage(BaseModel):
    model_config = ConfigDict(extra="ignore")
    members: list[Member] = []
    next_cursor: Optional[str] = None


class PostPage(BaseModel):
    model_config = ConfigDict(extra="ignore")
    posts: list[Post] = []
    next_cursor: Optional[str] = None


def _build_included_index(included: list[dict]) -> dict:
    return {(item["type"], item["id"]): item for item in included}


def parse_resource(item: dict, included: list[dict] = None) -> dict:
    result = {"id": item["id"], **item.get("attributes", {})}

    if included and "relationships" in item:
        rels = item["relationships"]
        index = _build_included_index(included)

        if "currently_entitled_tiers" in rels:
            tier_ids = [t["id"] for t in rels["currently_entitled_tiers"].get("data", [])]
            tier_names = []
            for tid in tier_ids:
                inc = index.get(("tier", tid))
                if inc:
                    tier_names.append(inc.get("attributes", {}).get("title", tid))
            result["tiers"] = tier_names

        if "user" in rels:
            user_data = rels["user"].get("data")
            if user_data:
                inc = index.get(("user", user_data["id"]))
                if inc:
                    result["user_name"] = inc.get("attributes", {}).get("full_name")

    return result


def parse_list(response: dict, model_class):
    included = response.get("included", [])
    items = []
    for item in response.get("data", []):
        parsed = parse_resource(item, included)
        items.append(model_class(**parsed))

    pagination = response.get("meta", {}).get("pagination", {})
    cursor = pagination.get("cursors", {}).get("next")

    return items, cursor


def parse_single(response: dict, model_class):
    data = response.get("data")
    if not data:
        raise ValueError("Invalid API response: missing 'data' key")
    included = response.get("included", [])
    parsed = parse_resource(data, included)
    return model_class(**parsed)



from mcp.server.fastmcp import FastMCP
from utils.client import PatreonClient

mcp = FastMCP(
    name="patreon",
    instructions=(
        "Patreon API integration. Workflow: "
        "1) Call fetch_campaigns to get campaign IDs. "
        "2) Use campaign_id with fetch_campaign, fetch_members, or fetch_posts. "
        "3) For paginated tools (fetch_members, fetch_posts), pass next_cursor as cursor to get more pages. "
        "Note: patron emails are not fetched for privacy reasons."
    ),
)
client = PatreonClient()



import os
import re
import httpx


class PatreonAPIError(Exception):
    def __init__(self, status_code: int, message: str):
        self.status_code = status_code
        super().__init__(f"Patreon API error {status_code}: {message}")


_ID_PATTERN = re.compile(r"^[0-9]{1,20}$")


def _validate_id(value: str, name: str = "ID") -> str:
    if not _ID_PATTERN.match(value):
        raise ValueError(f"Invalid {name}: must be 1-20 ASCII digits")
    return value


def _validate_cursor(value: str) -> str:
    if len(value) > 512 or not value.isprintable():
        raise ValueError("Invalid pagination cursor")
    return value


class PatreonClient:
    BASE_URL = "https://www.patreon.com/api/oauth2/v2"

    def __init__(self, access_token: str = None):
        token = access_token or os.getenv("PATREON_ACCESS_TOKEN")
        if not token:
            raise ValueError("PATREON_ACCESS_TOKEN is required")

        self.client = httpx.Client(
            base_url=self.BASE_URL,
            headers={
                "Authorization": f"Bearer {token}",
                "User-Agent": "PatreonMCP/1.0",
            },
            timeout=30.0,
            follow_redirects=False,
        )

    def _get(self, path: str, params: dict = None) -> dict:
        try:
            response = self.client.get(path, params=params)
            if not response.is_success:
                code = response.status_code
                if code == 429:
                    retry = response.headers.get("Retry-After", "60")
                    raise PatreonAPIError(429, f"Rate limited. Retry after {retry}s")
                if code == 401:
                    raise PatreonAPIError(401, "Unauthorized  - check your access token")
                if code == 403:
                    raise PatreonAPIError(403, "Forbidden  - insufficient permissions")
                if code == 404:
                    raise PatreonAPIError(404, "Resource not found")
                if 300 <= code < 400:
                    raise PatreonAPIError(code, "Unexpected redirect")
                raise PatreonAPIError(code, "Request failed")
        except PatreonAPIError:
            raise
        except httpx.RequestError:
            raise PatreonAPIError(0, "Connection error  - could not reach Patreon API")
        return response.json()

    def _build_fields(self, fields: dict[str, list[str]]) -> dict:
        return {f"fields[{k}]": ",".join(v) for k, v in fields.items()}

    def get_identity(self) -> dict:
        params = self._build_fields({
            "user": [
                "email", "full_name", "image_url", "about", "created",
                "url", "social_connections",
            ],
        })
        params["include"] = "campaign,memberships"
        return self._get("/identity", params)

    def get_campaigns(self) -> dict:
        params = self._build_fields({
            "campaign": [
                "creation_name", "patron_count", "pledge_url",
                "published_at", "url", "vanity", "is_monthly",
                "one_liner", "pay_per_name",
            ],
        })
        return self._get("/campaigns", params)

    def get_campaign(self, campaign_id: str) -> dict:
        _validate_id(campaign_id, "campaign_id")
        params = self._build_fields({
            "campaign": [
                "creation_name", "patron_count", "pledge_url",
                "published_at", "url", "vanity", "is_monthly",
                "created_at", "image_url", "summary",
                "one_liner", "pay_per_name",
            ],
            "tier": [
                "title", "amount_cents", "description",
                "published", "patron_count",
            ],
        })
        params["include"] = "tiers,creator"
        return self._get(f"/campaigns/{campaign_id}", params)

    def get_members(self, campaign_id: str, cursor: str = None) -> dict:
        _validate_id(campaign_id, "campaign_id")
        params = self._build_fields({
            "member": [
                "full_name", "patron_status",
                "pledge_cadence", "lifetime_support_cents",
                "currently_entitled_amount_cents", "last_charge_date",
                "last_charge_status", "will_pay_amount_cents",
                "is_follower",
            ],
            "tier": ["title", "amount_cents", "description"],
            "user": ["full_name", "image_url", "url"],
        })
        params["include"] = "currently_entitled_tiers,user"
        params["page[count]"] = "100"
        if cursor:
            _validate_cursor(cursor)
            params["page[cursor]"] = cursor
        return self._get(f"/campaigns/{campaign_id}/members", params)

    def get_posts(self, campaign_id: str, cursor: str = None) -> dict:
        _validate_id(campaign_id, "campaign_id")
        params = self._build_fields({
            "post": [
                "title", "content", "is_paid", "is_public",
                "published_at", "url", "embed_data", "embed_url",
                "app_id", "app_status",
            ],
        })
        params["page[count]"] = "20"
        if cursor:
            _validate_cursor(cursor)
            params["page[cursor]"] = cursor
        return self._get(f"/campaigns/{campaign_id}/posts", params)

    def get_post(self, post_id: str) -> dict:
        _validate_id(post_id, "post_id")
        params = self._build_fields({
            "post": [
                "title", "content", "is_paid", "is_public",
                "published_at", "url", "embed_data", "embed_url",
                "app_id", "app_status",
            ],
        })
        return self._get(f"/posts/{post_id}", params)
